"""A list for showing triggered keys."""

from .keys_display import KeysDisplay


class TriggeredKeys(KeysDisplay):
    """A widget to show the keys that have been triggered."""

    BORDER_TITLE = "Triggered Keys"
